
# q-expansion of newform 5328.2.a.g, downloaded from the LMFDB on 05 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5328
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [4], [-4], [-2], [-6], [6], [-8], [6], [-2], [-1], [0], [10], [12], [4], [4], [10], [4], [12], [-10], [-10], [0], [-2], [-2], [0], [-6], [12], [-10], [6], [8], [12], [12], [-4], [-8], [20], [2], [6], [0], [12], [-12], [2], [-8], [-14], [24], [-2], [4], [-16], [4], [10], [0], [0], [14], [20], [6], [20], [-24], [-20], [-18], [30], [26], [28], [32], [0], [34], [-20], [-10], [26], [-20], [6], [-10], [0], [16], [-22], [24], [-32], [-14], [-2], [18], [-34], [-16], [30], [-24], [-30], [14], [-16], [6], [30], [-30], [10], [-4], [-8], [30], [16], [6], [32], [-28], [24], [-14], [-34], [22], [-30], [36], [6], [8], [18], [-28], [16], [20], [14], [-22], [-6], [-4], [16], [-38], [-40], [10], [8], [18], [-12], [26], [6], [4], [12], [20], [-6], [-26], [24], [-34], [-50], [24], [-36], [8], [-2], [-20], [2], [-24], [44], [-2], [-22], [16], [28], [16], [44], [-10], [52], [-46], [6], [-22], [24], [2], [28], [-6], [-52], [26], [16], [-38], [24], [22], [-28], [-12], [-20], [2], [-16], [-38], [36], [-42], [-6], [-62], [30], [-28], [-14], [16], [-42], [32], [50], [-14], [64], [-52], [18], [18], [12], [30], [6], [44], [28], [30], [-26], [14], [-56], [-18], [-12], [-2], [-60], [24], [-44], [26], [6], [40], [24], [6], [16], [42], [-34], [-68], [-30], [-16], [0], [-18], [40], [10], [-6], [-26], [-24], [-32], [62], [-22], [66], [16], [4], [-42], [28], [56], [34], [28], [66], [-12], [-40], [-24], [-52], [14], [4], [4], [4], [48], [-72], [50], [-42], [-60], [72], [-4], [-34], [28], [10], [-52], [24], [-38], [12], [-4], [-16], [-50], [20], [-24], [-10], [-18], [12], [-10], [-28], [68], [-62], [-38], [-44], [66], [-26], [-6], [-56], [-60], [-70], [44], [-10], [26], [-10], [-26], [-22], [82], [24], [40], [-22], [-66], [-4], [0], [-78], [24], [30], [22], [20], [-10], [16], [34], [6], [30], [-20], [36], [72], [22], [16], [-32], [-12], [76], [8], [50], [20], [12], [-36], [74], [46], [76], [6], [-48], [58], [-16], [-4], [-68], [-12], [-72], [-18], [-44], [40], [14], [-54], [50], [78], [26], [-36], [44], [6], [28], [-10], [-24], [0], [-70], [10], [-2], [12], [-58], [-20], [-74], [-48], [-58], [72], [-18], [-88], [-66], [14], [-38], [36], [34], [56], [0], [8], [56], [70], [6], [46], [74], [12], [0], [-84], [56], [-32], [34], [-36], [-72], [60], [64], [34], [78], [-46], [-78], [68], [-10], [-4], [-48], [6], [-98], [52], [-12], [46], [-98], [22], [30], [58], [-2], [-28], [-58], [12], [8], [-42], [46], [48], [-2], [34], [36], [-42], [12], [86], [-74], [36], [-8], [-56], [66], [50], [94], [78], [-94], [88], [38], [-96], [-4], [64], [66], [0], [20], [2], [-50], [92], [24], [-52], [60], [-64], [-26], [-6], [24], [-76], [-18], [72], [44], [12], [-16], [24]]
